﻿#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.

		
	Function CorrelateCore2Log( )
		SetDataFolder root:
			String/G G_LabHole, G_LabTechnique, G_LabData, G_LabScale, G_LogHole, G_LogTechnique, G_LogData, G_LogScale, G_YorN
			Variable/G G_IntervalAdd, G_Core
			String LabHole = G_LabHole, LabTechnique = G_LabTechnique, LabData = G_LabData, LabScale = G_LabScale, YorN = G_YorN
			String LogHole = G_LogHole, LogTechnique = G_LogTechnique, LogData = G_LogData, LogScale = G_LogScale
			Variable IntervalAdd = G_IntervalAdd, Core = G_Core
			Prompt Core, "Core Number"
			Prompt LabHole, "Lab Hole"
			Prompt LabTechnique, "Lab Technique"
			Prompt LabData, "Lab Data"
			Prompt LabScale, "Lab Depth Scale ( e.g. CSF )"
			Prompt LogHole, "Log Hole"
			Prompt LogTechnique, "LogTechnique"
			Prompt LogData, "Log Data"
			Prompt LogScale, "Log Depth Scale ( e.g. WMSF )"
			Prompt IntervalAdd, "Extra increment (m) to add to search interval of log data"
			Prompt YorN, "Do you want to suppress printing of the command line version of this function?", popup, "Y;N"
			DoPrompt "CorrelateCore2Log", Core, LabHole, LabTechnique, LabData, LabScale, LogHole, LogTechnique, LogData, LogScale, IntervalAdd
			G_LabHole = LabHole; G_LabTechnique = LabTechnique; G_LabData = LabData; G_LabScale = LabScale
			G_LogHole = LogHole; G_LogTechnique = LogTechnique; G_LogData = LogData; G_LogScale = LogScale
			G_IntervalAdd = IntervalAdd; G_Core = Core; G_YorN = YorN
			AbortOnValue V_Flag, 0	
			DoPrompt "CorrelateCore2Log", YorN
			AbortOnValue V_Flag, 0	
			CorrelateCore2Log_F( Core, LabHole, LabTechnique, LabData, LabScale, LogHole, LogTechnique, LogData, LogScale, IntervalAdd )	
			String S1 = "CorrelateCore2Log_F( " + Num2Str( Core ) + ", \"" + LabHole + "\", " + "\"" + LabTechnique + "\", \"" + LabData + "\", \"" + LabScale + "\", "
			String S2 = "\"" + LogHole + "\", \"" + LogTechnique + "\", \"" + LogData + "\", \"" + LogScale + "\", " + Num2Str( IntervalAdd ) + " )"	
			If( StringMatch( YorN, "N" ) )
				Print S1, S2
			EndIf
	End

---------------------------------------------------------------
		
	Function CorrelateCore2Log_F( Core, LabHole, LabTechnique, LabData, LabScale, LogHole, LogTechnique, LogData, LogScale, IntervalAdd )
		String LabHole, LabTechnique, LabData, LabScale, LogHole, LogTechnique, LogData, LogScale
		Variable IntervalAdd, Core
		SetDataFolder root:
			SelectCore_F( LabHole, Core, LabTechnique, LabData, LabScale )
			Variable IntervalTop, IntervalBottom, Points, Offset, InterpResolution = .01
			String Site = RemoveEnding( LabHole )
			If( !Exists( LabHole + "_" + LabTechnique + "_LogOffsets" )	)			
				Make/O/N = 100 $LabHole + "_" + LabTechnique + "_LogOffsets"
				WAVE LogOffsets = $LabHole + "_" + LabTechnique + "_LogOffsets"
				LogOffsets = NaN
			Else
				WAVE LogOffsets = $LabHole + "_" + LabTechnique + "_LogOffsets"
			EndIf
		SetDataFolder $"root:" + Site + "_" + LogTechnique
			WAVE LogDataWv = $LogHole + "_" + LogTechnique + "_" + LogData
			WAVE LogDepthWv = $LogHole + "_" + LogTechnique + "_" + LogScale
		SetDataFolder root:Xtras // The first thing to do is interpolate both data to the same depth scale
			WAVE CoreData, CoreDepth // These waves were formed by running Select_Core in the code above
			DeletePoints 0,1, CoreData, CoreDepth // This line and the next delete the first and last points of the core data wave in case there are end effects
			DeletePoints ( NumPnts( CoreDepth ) - 1 ),1, CoreData, CoreDepth					
			If( WaveMax( CoreDepth ) > WaveMax( LogDepthWv ) )
				SetDataFolder root:
				ABORT "The depth to the bottom of the core exceeds the depth to the bottom of the well log. No Overlap"
			EndIf
			If( WaveMin( CoreDepth ) < WaveMin( LogDepthWv ) )
				SetDataFolder root:
				ABORT "The depth to the top of the core is less than the depth to the top of the well log. No Overlap"
			EndIf
 			IntervalBottom = CoreDepth[ NumPnts( CoreDepth ) - 1 ]
			IntervalTop = CoreDepth[ 0 ]
			Make/O/N = ( ( ( IntervalBottom - IntervalTop ) / InterpResolution ) + 1 ) CoreDataI, CoreDepthI // These are the interpolated data waves
			SetScale/P x, IntervalTop, InterpResolution, CoreDepthI, CoreDataI
			CoreDepthI = x
			CoreDataI = Interp( CoreDepthI, CoreDepth, CoreData )
			IntervalTop -= IntervalAdd // The search interval in the logs is set wider by IntervalAdd m top and bottom than the interval for the core data
			IntervalBottom += IntervalAdd
			If( IntervalTop < 0 )
				IntervalTop = 0
			EndIf
			Make/O/N = ( ( ( IntervalBottom - IntervalTop ) / InterpResolution ) + 1 ) LogDataI, LogDepthI // These are the interpolated log waves
			SetScale/P x, IntervalTop, InterpResolution, LogDepthI, LogDataI
			LogDepthI = x
			LogDataI = Interp( LogDepthI, LogDepthWv, LogDataWv )
			Duplicate/O LogDataI Correlation
			Correlate/NODC/AUTO CoreDataI, Correlation
			Variable CorrMax = WaveMax( Correlation )
			Correlation /= CorrMax
			SetScale/P x DimOffset( Correlation, 0 ) - CoreDepth[ 0 ], InterpResolution,"", Correlation		
			FindPeaks( Correlation )
			WAVE PeakList, PeakAmpList
			Offset = PeakList[ 0 ]
			LogOffsets[ Core ] = Offset
			If( StrSearch( winlist( "*", ";", "VISIBLE:1,WIN:1" ), "CorrelationPlot", 0, 2 ) < -.01 )
				EXECUTE "CorrelationPlot()"
			Else
				DoWindow/F CorrelationPlot
			EndIf
			If( StrSearch( winlist( "*", ";", "VISIBLE:1,WIN:2" ), "PeakListTable", 0, 2 ) < -.01 )
				EXECUTE "PeakListTable()"
			Else
				DoWindow/F PeakListTable
			EndIf
			If( StrSearch( winlist( "*", ";", "VISIBLE:1,WIN:1" ), "Lab2LogDataPlot", 0, 2 ) < -.01 )
				EXECUTE "Lab2LogDataPlot()"
			Else
				DoWindow/F Lab2LogDataPlot
			EndIf
			ModifyGraph offset( CoreDataI ) = { Offset, 0 }
			Printf "\t\t\t\t%s %d %s %0.2f\r", "Initial Offset for Core", Core, "=", Offset
			KillWaves/Z CoreDepth, CoreData, Correlation_DIF, Data_Dif
			KillWaves/Z W_FindLevels, ABSPeaklist, CoreDepthI, LogDepthI
		SetDataFolder root:
	End
		
---------------------------------------------------------------
	
	Function ChooseOffset()
		SetDataFolder root:
			NVAR G_Core
			SVAR G_LabHole, G_LabTechnique
			WAVE OffsetWv = $G_LabHole + "_" + G_LabTechnique + "_LogOffsets"
		SetDataFolder root:Xtras
			Variable/G G_PeakNumber 
			Variable PeakNumber = G_PeakNumber
			Make/O/N = 100 $G_LabHole + "_" + G_LabTechnique + "_LogOffsets"
			WAVE PeakList, CoreDataI
			DoWindow/F Lab2LogDataPlot
			SetAxis/A
			Prompt PeakNumber, "Peak Number To Use For Offset"
			DoPrompt "ChooseOffset", PeakNumber
			G_PeakNumber = PeakNumber
			AbortOnValue V_Flag, 0
			Variable Offset = PeakList[ PeakNumber ]
			ModifyGraph offset( CoreDataI ) = { Offset, 0 }
			OffsetWv[ G_Core ] = Offset
		SetDataFolder root:
	End
	
---------------------------------------------------------------
			
Window Lab2LogDataPlot() : Graph
	PauseUpdate; Silent 1		// building window...
	String fldrSav0= GetDataFolder(1)
	SetDataFolder root:Xtras:
	Display /W=(577,58,1358,463)/L=CoreData CoreDataI as "DataPlot"
	AppendToGraph/R=LogData LogDataI
	SetDataFolder fldrSav0
	ModifyGraph lSize=2
	ModifyGraph rgb(CoreDataI)=(0,0,0)
	ModifyGraph offset(CoreDataI)={2.34432860251231,-0.975312200085874}
	ModifyGraph grid(bottom)=1
	ModifyGraph nticks(bottom)=10
	ModifyGraph minor(bottom)=1
	ModifyGraph lblPos(CoreData)=44,lblPos(LogData)=45
	ModifyGraph lblLatPos(CoreData)=3,lblLatPos(LogData)=-1
	ModifyGraph freePos(CoreData)=0
	ModifyGraph freePos(LogData)=0
	Label CoreData "\\Z14CoreData"
	Label LogData "\\Z14LogData"
	ShowInfo
	SetAxis/A
	Tag/C/N=text0/M/X=-9.29/Y=-11.21/L=0 CoreDataI, 41.990000000000002, "\\Z14Offset = \\Ox"
	Legend/C/N=text1/J/M/A=MC/X=-25.71/Y=37.74 "\\s(CoreDataI) CoreDataI\r\\s(LogDataI) LogDataI"
EndMacro
		
	---------------------------------------------------------------
			
	Window CorrelationPlot() : Graph
		PauseUpdate; Silent 1		// building window...
		String fldrSav0= GetDataFolder(1)
		SetDataFolder root:Xtras:
		Display /W=(615,569,1124,804) Correlation as "CorrelationPlot"
		SetDataFolder fldrSav0
		ModifyGraph margin(right)=7
		ModifyGraph grid(bottom)=1
		ModifyGraph nticks(bottom)=10
		ModifyGraph minor(bottom)=1
		ModifyGraph gridRGB(bottom)=(0,0,0)
		ModifyGraph gridStyle(bottom)=1
		Label bottom "\\Z14Offset"
		ShowInfo
	EndMacro
		
	---------------------------------------------------------------
			
Window PeakListTable() : Table
	PauseUpdate; Silent 1		// building window...
	String fldrSav0= GetDataFolder(1)
	SetDataFolder root:Xtras:
	Edit/W=(1154,550,1381,893) PeakList as "PeakList"
	ModifyTable alignment=1,format(Point)=1,format(PeakList)=3
	SetDataFolder fldrSav0
EndMacro
	
