﻿#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.

// This function performs a lighting correction core by core using the lightness values of HSL versions of the images. It forms "C" section images for use in core image assembly

	Function LightnessLightingCorrection( )
			Variable i
			String KList = WaveList( "K_*", ";", "" ), SectionImage
			For( i = 0; i < ItemsInList( KList ); i += 1 ) // This loop duplicates the "K_" images as "C_" images
				SectionImage = StringFromList( i, KList )
				Duplicate/O $SectionImage, $ReplaceString( "K_", SectionImage, "C_" )  
			EndFor
			String CList = WaveList( "C_*", ";", "" ) // List of "C_" images for processing
			Make/O/N = 2 XPoints, YPoints = { .5, .5 }; // Y points are the Y coordinates of the profile end points
			Make/O/N = 0 LightnessProfiles, LightnessX, Mask255
			For( i = 0; i < ItemsInList( CList ); i += 1 ) // This loop extracts and concatenates lightness curves for each section of a core
				WAVE C_Wv = $StringFromList( i, CList )
				XPoints = { LeftX( C_Wv ), RightX( C_Wv ) } // X End points for the lightness profile
				ImageTransform/O RGB2HSL C_Wv // Changes the "C_" image to HSL
				ImagelineProfile/P = 2 XWave = XPoints, yWave = YPoints, srcwave = C_Wv, Width = 90 // Extracts an "L" profile ( /P=2 ) that uses 90 pixels ( width ) normal to the profile line
				WAVE W_ImageLineProfile, W_LineProfileX // The profile - X and Y values in pixels
				Concatenate/NP { W_ImageLineProfile }, LightnessProfiles // Adds profiles to cumulative wave
				Concatenate/NP { W_LineProfileX }, LightnessX
			EndFor	
			Duplicate/O LightnessProfiles, LightnessResiduals
			CurveFit/Q poly 3, LightnessProfiles /X=LightnessX /D /R=LightnessResiduals // Fit a curve to the lightness data and make a residual curve as well
			WAVE W_Coef, fit_LightnessProfiles
			WaveStats/Q LightnessResiduals // Need stdev value for residuals
			LightnessProfiles = LightnessResiduals > ( V_sdev * 2 ) || LightnessResiduals < ( V_sdev * -2 ) ? NaN : LightnessProfiles // Set to NaN all values outside of 2 sigma of the residual curve
			CurveFit/Q poly 3, LightnessProfiles /X=LightnessX /D // New curve fit with outliers removed
			Make/O/N = ( WaveMax( LightnessX ) ) CorrectionWave // Make a correction wave as long as the longest section ( in pixels )
			CorrectionWave = WaveMax( fit_LightnessProfiles ) - ( W_Coef[ 0 ] + W_Coef[ 1 ] * X + W_Coef[ 2 ] * X * X ) // The correction is the difference between the fit line and its max value
			For( i = 0; i < ItemsInList( CList ); i += 1 ) // This loop adds the lighting correction to each "C" section 
				WAVE C_Wv = $StringFromList( i, CList )
				ReDimension/S C_Wv, CorrectionWave // Temporarily make the image floating point so that values can exceed 255
				C_Wv[][][2] += CorrectionWave[ p ] // Correction added
				Duplicate/O/R = [][][2] C_Wv Mask255
				Redimension/B/U C_Wv // Set back to unsigned byte format
				ImageTransform/O hsl2rgb C_Wv // xform back to RGB
				C_Wv /= 256 // Adjustment for 16 bit format that is output from ImageTransform
				Redimension/B/U C_Wv // Set to unsigned byte format
				C_Wv[][][0] = Mask255[p][q] > 254 ? 254 : C_Wv[p][q][0] // These lines replace spots where the correction causes
				C_Wv[][][1] = Mask255[p][q] > 254 ? 254 : C_Wv[p][q][1] //  the unsigned byte values to be > 254 or < 1
				C_Wv[][][2] = Mask255[p][q] > 254 ? 254 : C_Wv[p][q][2] // 
				C_Wv[][][0] = Mask255[p][q] < 1 ? 1 : C_Wv[p][q][0] // 
				C_Wv[][][1] = Mask255[p][q] < 1 ? 1 : C_Wv[p][q][1] // 
				C_Wv[][][2] = Mask255[p][q] < 1 ? 1 : C_Wv[p][q][2] // 
				KillWaves/Z Mask255
			EndFor	
			KillWaves/Z CorrectionWave, fit_LightnessProfiles, LightnessProfiles, LightnessResiduals, LightnessX
			KillWaves/Z W_Coef, W_ImageLineProfile, W_LineProfileDisplacement, W_LineProfileX, W_LineProfileY
			KillWaves/Z W_ParamConfidenceInterval, W_sigma, XPoints, YPoints,Mask255
	End
	
	
	
//	Function LightnessLightingCorrection( )
			Variable i
			String KList = WaveList( "K_*", ";", "" ), SectionImage
			For( i = 0; i < ItemsInList( KList ); i += 1 ) // This loop duplicates the "K_" images as "C_" images
				SectionImage = StringFromList( i, KList )
				Duplicate/O $SectionImage, $ReplaceString( "K_", SectionImage, "C_" )  
			EndFor
			String CList = WaveList( "C_*", ";", "" ) // List of "C_" images for processing
			Make/O/N = 2 XPoints, YPoints = { .5, .5 }; // Y points are the Y coordinates of the profile end points
			Make/O/N = 0 LightnessProfiles, LightnessX
			For( i = 0; i < ItemsInList( CList ); i += 1 ) // This loop extracts and concatenates lightness curves for each section of a core
				WAVE C_Wv = $StringFromList( i, CList )
				XPoints = { LeftX( C_Wv ), RightX( C_Wv ) } // X End points for the lightness profile
				ImageTransform/O RGB2HSL C_Wv // Changes the "C_" image to HSL
				ImagelineProfile/P = 2 XWave = XPoints, yWave = YPoints, srcwave = C_Wv, Width = 90 // Extracts an "L" profile ( /P=2 ) that uses 90 pixels ( width ) normal to the profile line
				WAVE W_ImageLineProfile, W_LineProfileX // The profile - X and Y values in pixels
				Concatenate/NP { W_ImageLineProfile }, LightnessProfiles // Adds profiles to cumulative wave
				Concatenate/NP { W_LineProfileX }, LightnessX
			EndFor	
			Duplicate/O LightnessProfiles, LightnessResiduals
			CurveFit/Q poly 3, LightnessProfiles /X=LightnessX /D /R=LightnessResiduals // Fit a curve to the lightness data and make a residual curve as well
			WAVE W_Coef, fit_LightnessProfiles
			WaveStats/Q LightnessResiduals // Need stdev value for residuals
			LightnessProfiles = LightnessResiduals > ( V_sdev * 2 ) || LightnessResiduals < ( V_sdev * -2 ) ? NaN : LightnessProfiles // Set to NaN all values outside of 2 sigma of the residual curve
			CurveFit/Q poly 3, LightnessProfiles /X=LightnessX /D // New curve fit with outliers removed
			Make/O/N = ( WaveMax( LightnessX ) ) CorrectionWave // Make a correction wave as long as the longest section ( in pixels )
			CorrectionWave = WaveMax( fit_LightnessProfiles ) - ( W_Coef[ 0 ] + W_Coef[ 1 ] * X + W_Coef[ 2 ] * X * X ) // The correction is the difference between the fit line and its max value
			For( i = 0; i < ItemsInList( CList ); i += 1 ) // This loop adds the lighting correction to each "C" section 
				WAVE C_Wv = $StringFromList( i, CList )
				Redimension/B/U C_Wv // Set to unsigned byte format
				C_Wv[][][2] += CorrectionWave[ p ] // Correction added
				ImageTransform/O hsl2rgb C_Wv // xform back to RGB
				C_Wv /= 256 // Adjustment for 16 bit format that is output from ImageTransform
				Redimension/B/U C_Wv // Set to unsigned byte format
			EndFor	
			KillWaves/Z CorrectionWave, fit_LightnessProfiles, LightnessProfiles, LightnessResiduals, LightnessX
			KillWaves/Z W_Coef, W_ImageLineProfile, W_LineProfileDisplacement, W_LineProfileX, W_LineProfileY
			KillWaves/Z W_ParamConfidenceInterval, W_sigma, XPoints, YPoints
	End