#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.

// New version for data splicing. Created JUN2018.
//

	Function SpliceData()
		SetDataFolder root:
			Variable i, j, NumberOfWaves
			String/G G_Site, G_Technique, G_DepthType
			String Site = G_Site,  Technique = G_Technique, DepthType = G_DepthType
			Prompt Site, "Site"
			Prompt Technique, "Technique"
			Prompt DepthType, "Depth type ( e.g. CCSF, mcd, rmcd, CCSFs ... )"
			DoPrompt "SpliceData", Site, Technique, DepthType
			G_Site = Site; G_Technique = Technique	; G_DepthType = DepthType
			AbortOnValue V_Flag, 0
			SpliceData_F( Site, Technique, DepthType )
			Print "SpliceData_F( \"" + Site + "\", \"" + Technique + "\", \"" + DepthType + "\" )"
	End

	Function SpliceData_F( Site, Technique, DepthType )
			String Site, Technique, DepthType
		SetDataFolder root:
			String/G G_Site, G_Technique, G_DepthType
			String SplWvName, DataWvName
			Variable i, j, NumberOfWaves
			CheckEqualWaveLengthsSite( Site, Technique )
			String MatrixList = WaveList( "*_Coring_Matrix", ";", "" )
			For( i = 0; i < ItemsInList( MatrixList ); i += 1 )
				CheckMatrix4CCSF( DataWaveHole( StringFromList( i, MatrixList ) ) )
			EndFor
			G_Site = Site; G_Technique = Technique	; G_DepthType = DepthType
			DFREF SpliceFldr = $"root:" + Site + "_Splice_Intervals" // The name of the splice interval folder made into a folder reference
			DFREF DataFldr = $"root:" + Site + "_" + Technique // The name of the data folder made into a folder reference
		 	If( !DataFolderExists( "root:" + Site + "_Splice_Intervals" ) ) // Check to see if Spliced Intervals folder exists
		 		ABORT "This function uses the \"Splice_Intervals\" form of splice data. Use the \"Splice_Tie_Data_2_Intervals\" function to form an Intervals folder from a Ties folder."
		 	EndIf
		 	If( !DataFolderExists( "root:" + Site + "_" + Technique ) ) // Check to see if data folder exists
		 		ABORT "I can't find the data folder \"" + Site + "_" + Technique + "\"."
		 	EndIf
			If( SpliceDataCheck( Site, "Splice_Intervals" ) ) // This is a separate function to check the splice tables for lengths of waves.
				Abort
			Endif
			CheckSiteWavesAreText( Site, Technique ) //Checks to see that all Site waves are either TEXT or Numbers. Error will occur if this isn't fixed.
		SetDataFolder SpliceFldr
			WAVE CCSF_Top, CCSF_Bottom // Sets wave references to the interval depths
			WAVE/T Hole_Top // Sets wave reference to the Hole

		SetDataFolder DataFldr
			If( StrLen( WaveList( "*_Index", ";", "" ) ) > 0 ) // Checks to see if there is an "_Index" wave left from older splicing functions - must be removed.
				SetDataFolder root:
				ABORT "There is an \"_Index\" splice wave in the " + Technique + " data folder.\r\rThey are no longer used and need to be deleted before running this function."
			EndIf
			String HoleList = WaveList( "*" + DepthType, ";", "" ), HoleLetter, WvList, WvList2, WaveCountMsg = "\r" // HoleList is actually a list of depth waves
			Variable WaveCount
			For( i = 0; i < ItemsInList( HoleList ); i += 1 ) // This loop checks to see if all of the Holes have the same number of waves
				HoleLetter = StringFromList( i, HoleList )[ Strlen( Site ) ] // HoleLetter is grabbed from the depth wave names
				If( !StringMatch( HoleLetter, "S" ) && !StringMatch( HoleLetter, "_" ) ) // Skips existing spliced data when checking. The second conditional is for the old "_Splice_" format of spliced waves.
					WvList = WaveList( Site + HoleLetter + "*", ";", "" ) // Creates a list of waves from 1 hole.
					WaveCount = ItemsInList( WvList )
					WaveCountMsg = WaveCountMsg + Site + HoleLetter + " has " + Num2Str( WaveCount ) + " Waves in " + Site + "_" + Technique + "\r"
					If( i == 0 )
					 	NumberOfWaves = ItemsInList( WvList ) // Sets NumberOfWaves to the number of waves from the first hole ( usually hole A )
					ElseIf( NumberOfWaves != ItemsInList( WvList ) ) // Compares the number of waves from the first hole to the number of waves from other holes
						SetDataFolder root: // If the numbers of waves aren't the same the function is aborted.
						Print WaveCountMsg
					 	ABORT "There are different numbers of waves for different Holes in the data folder. All Holes must have the same number and names of waves in order to splice all of the data.\r\r  See HISTORY for more information."
					EndIf
				EndIf
			EndFor
			String FirstHoleLetter = StringFromList( 0, HoleList )[ Strlen( Site ) ] // Gets letter of first hole
			WvList = SortList( WaveList( Site + FirstHoleLetter + "*", ";", "" ) ) // Creates a sorted list of wave names from the first hole.
			For( i = 0; i < ItemsInList( HoleList ); i += 1 ) // This loop checks to be sure that there are no spelling errors in the wavenames
				HoleLetter = StringFromList( i, HoleList )[ Strlen( Site ) ]
				If( !StringMatch( HoleLetter, "S" ) && !StringMatch( HoleLetter, "_" ) ) // Skips existing spliced data when checking
					WvList2 = SortList( WaveList( Site + HoleLetter + "*", ";", "" ) ) // WvList2 is a sorted list of each hole as the loop progresses
					WvList2 = ReplaceString( Site + HoleLetter, WvList2, Site + FirstHoleLetter ) // Site and Hole in WvList2 are replaced with the same from the first hole.
					If( CmpStr( WvList, WvList2 ) ) // Looks for typos in wave names by comparing first hole list and list2 ( with first hole substitution ).
						SetDataFolder root:
						ABORT "WaveName(s) of Hole " + HoleLetter + " data are spelled differently than for Hole " + FirstHoleLetter
					EndIf
				EndIf
			EndFor
		
			// This is the end of the checks of data. The next section creates the "S" splice waves, or zeroes out existing splice waves.
			
			String TextSpliceList = WaveList( Site + FirstHoleLetter + "*", ";", "TEXT:1" ) // These 4 lines create wavename lists for spliced text and numerical waves
			TextSpliceList = ReplaceString( Site + FirstHoleLetter, TextSpliceList, Site + "S" )
			String NumSpliceList = WaveList( Site + FirstHoleLetter + "*", ";", "TEXT:0" )
			NumSpliceList = ReplaceString( Site + FirstHoleLetter, NumSpliceList, Site + "S" )
			For( i = 0; i < ItemsInList( TextSpliceList ); i += 1 ) // These loops create, or zero out existing, spliced text and numerical waves
				Make/T/O/N = 0 $StringFromList( i, TextSpliceList )
			EndFor
			For( i = 0; i < ItemsInList( NumSpliceList ); i += 1 )
				Make/O/N = 0 $StringFromList( i, NumSpliceList )
			EndFor

			// This final section actually does the splicing. It first forms temporary waves in a temporary waves folder. The temp waves consist of all of the 
			// data from each hole included in the splice. The temp waves are then concatenated ( added together ) and sorted by depth.

			NewDataFolder/O $"root:TempWaves" // TempWaves are formed in a separate temporary folder so they don't mess up the data folder if the function doesn't run to completion.
			DFREF TempFldr = $"root:TempWaves"
			For( j = 0; j < ItemsInList( HoleList ); j += 1 ) // This loop runs once for each hole and creates Masks for each hole named MaskA, MaskB, etc
				HoleLetter = StringFromList( j, HoleList )[ Strlen( Site ) ] // Sets Holeletter as A, B, C, etc
				If( StringMatch( HoleLetter, "S" ) || StringMatch( HoleLetter, "_" ) ) // Checks for existing splice waves of old or new format
										// Skip spliced "S"  or "_Splice" waves
				Else
					WAVE DepthWv = $Site + HoleLetter + "_" + Technique + "_" + DepthType // Sets depth wave to input - mcd, ccsf, ccsfA, rmcd, etc
					Duplicate/O DepthWv Mask
					Mask = NaN // Mask is the same length as the data and starts out as all NaN
					For( i = 0; i < NumPnts( CCSF_Top ); i += 1 ) // This loop works through the splice interval table line by line
						If( StringMatch( HoleLetter, Hole_Top[ i ] ) ) // Processes only the Holeletter hole
							Mask = DepthWv >= CCSF_Top[ i ] && DepthWv <= CCSF_Bottom[ i ] ? DepthWv : Mask // Mask now contains depth values witin the splice intervals and nans outside the intervals
						EndIf
					EndFor
					Duplicate/O Mask $"Mask" + HoleLetter // Generic mask is duplicated as a wave named MaskA, MaskB, etc
				EndIf
			EndFor
			KillWaves Mask // Discard generic Mask wave - no longer needed
			String MaskList = WaveList( "Mask*", ";", "" ) // List of the mask waves
			For( i = 0; i < ItemsInList( MaskList ); i += 1 ) // This loop cycles through the mask for each hole
				WAVE HoleMask = $StringFromList( i, MaskList ) // Creates waveref for one of the masks
				HoleLetter = StringFromList( i, MaskList )[ StrLen( NameOfWave( HoleMask ) ) -1 ] // HoleLetter is grabbed from the mask name
				WvList = WaveList( Site + HoleLetter + "*" , ";", "" ) // A list of waves from 1 hole
				WaveStats/Q HoleMask // Wavestats is run to determine the number of NaNs in the mask
				For( j = 0; j < ItemsInList( WvList ); j += 1 ) // This loop works through each wave from the current hole
					WAVE Wv = $StringFromList( j, WvList ) // Sets a waveref for the wave
					Duplicate/O Wv $"root:TempWaves:" + StringFromList( j, WvList ) + "_Temp" // Duplicates the wave and adds "_Temp" to the wave name. Temp waves are stored in a temporary folder
					WAVE TmpWv = $"root:TempWaves:" + StringFromList( j, WvList ) + "_Temp" // Sets waveref for temporary wave
					Sort HoleMask, TmpWv // The temporary wave is sorted according the the depths and NaNs in the mask. Note that the mask is not sorted.
					ReDimension/N = ( V_Npnts ) TmpWv // The temporary wave is redimensioned to get rid of all NaNs, which are at the end of the wave after sorting.
				EndFor													//    The text waves like Hole or Type are simply truncated to the same length.
			EndFor 
			EXECUTE "KillWaves/Z " + WaveList( "Mask*", ",", "" ) // The masks are deleted
			String SpliceWaveList = WaveList( Site + "S*", ";", "" ) // A list of splice waves is formed
			HoleList = ReplaceString( "Mask", MaskList, Site ) // A list of holes is formed by replacing Mask with the site
		SetDataFolder TempFldr
			For( i = 0; i < ItemsInList( SpliceWaveList ); i += 1 ) // This loop works through the splice wave list
				String Data = DataWaveData( StringFromList( i, SpliceWaveList ) ) // Data is the data ( e.g. Hole, CCSF, mcd, etc ) being processed in the loop
				String TmpStr = WaveList( "*_" + Technique + "_" + Data + "_Temp", ";", "" ) // List of _Temp waves for 1 data type
				String SplStr = Site + "S_" + Technique + "_" + Data // The name of the splice wave for the data type being processed
				WAVE SplWave = DataFldr:$Site + "S_" + Technique + "_" + Data // Waveref is assigned to the splice wave
				Concatenate/NP TmpStr, SplWave // The temporary waves with the splice data for each hole are concatenated into the splice wave
			EndFor
		SetDataFolder DataFldr 
			EXECUTE "Sort " + Site + "S_" + Technique + "_" + DepthType + " " + WaveList( Site + "S*", ",", "" ) // All of the splice waves are sorted according to depth
		SetDataFolder root:
			KillDataFolder TempWaves // The temporary waves are discarded
	End


