﻿#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.


	Function SpliceImages( )
		SetDataFolder root:
			String/G G_Site
			String Site = G_Site, ImageFldrs, HoleLtr, TwoDigitCoreNumber
			Variable/G G_XScaling, ErrorMsg
			Variable XScaling = G_XScaling, i, j
			Prompt Site, "Site:"
			Prompt XScaling, "Depth Scaling (m):"
			DoPrompt "SpliceImages", Site, XScaling
			G_Site = Site; G_XScaling = XScaling
			AbortOnvalue V_flag, 0
			SpliceImages_F( Site, XScaling )
			Print "\rSpliceImages_F( \"" + Site + "\", " + Num2Str( XScaling ) + " )"
	End

	Function SpliceImages_F( Site, XScaling )
			String Site
			Variable XScaling
		SetDataFolder root:
			String ImageFldrs, HoleLtr, TwoDigitCoreNumber
			Variable/G ErrorMsg
			Variable i, j
			If( !DataFolderExists( Site + "_Splice_Intervals" ) ) // Checks for Splice Interval folder.
				ABORT "The splce interval folder " + Site + "_Splice_Intervals is missing.\r\r  This function only uses Intervals, not Ties."
			Endif
			DFREF IntervalTable = $Site + "_Splice_Intervals"
			If( SpliceDataCheck( Site, "Splice_Intervals" ) ) // This is a separate function to check the splice table.
				Abort
			Endif
			If( CheckImages4Splicing( Site, "Splice_Intervals" ) ) // Returns "1" if the splice interval depths fall outside the depths of the appropriate core.
				DoAlert/T = "Image Splicing Problem" 1, "There is a problem with depths of the splice interval versus top and bottom depths of a core image.\r\rSee History for details.\r\rDo you want to continue? Splice will exclude problem cores."
				AbortonValue V_Flag - 1, 0
			EndIf
		SetDataFolder IntervalTable
			WAVE Core_Top, CCSF_Top, CCSF_Bottom // Identifies the important numerical waves in the splice interval tale
			WAVE/T Hole_Top // Identifies the important text wave in the splice interval tale
		SetDataFolder root:
			Make/B/O/N = ( (WaveMax( CCSF_Bottom ) + XScaling) / XScaling, 101, 3 ) $Site + "_Splice" // Creates an empty spliced image based on max interval depth.
			WAVE Splice = $Site + "_Splice"
			SetScale/P y 0,.01,"", Splice
			Splice = NaN
			SetScale/P x 0, XScaling, "m", Splice
			For( i = 0; i < NumPnts( Core_Top ); i += 1 )
				If( Mod( i, 5 ) == 0 )
					Printf " %d ", i
				EndIf
				SPrintf TwoDigitCoreNumber, "%03.0f", Core_Top[ i ]
				TwoDigitCoreNumber = TwoDigitCoreNumber[ 1, inf ]
				WAVE/Z CoreImage = $"root:" + Site + Hole_Top[ i ] + "_S2:S2_" + Site + Hole_Top[ i ] + "_" + TwoDigitCoreNumber
				If( numpnts( CoreImage ) == 0 )
				Else
					ImageInterpolate/S = { CCSF_Top[ i ], XScaling, CCSF_Bottom[ i ], 0, .01, 1 } Bilinear CoreImage
					WAVE M_InterpolatedImage
					ReDimension/B/U M_InterpolatedImage
					  EXECUTE  Site + "_Splice( LeftX(M_InterpolatedImage),RightX(M_InterpolatedImage) )()() = M_InterpolatedImage(x)(y)(z)"
					  ErrorMsg = GetRTError( 1 ) // This line suppresses error messages associated with rounding errors on the top and bottom depths of the splice section.
				EndIf
			EndFor
		SetDataFolder root:
		KillWaves/Z M_InterpolatedImage
	End
